#include "SiteServer.h"

#include "Things/TemperatureSensor.h"

#if !defined(NO_STD)
#include <functional>
#include <memory>
#endif

namespace RoboidControl {

SiteServer::SiteServer(int port) {
  this->name = "Site Server";
  this->publishInterval = 0;

  this->ipAddress = "0.0.0.0";
  this->port = port;

  SetupUDP(port, ipAddress, 0);
}

void SiteServer::UpdateMyThings(unsigned long currentTimeMs) {
  for (Thing* thing : this->things) {
    if (thing == nullptr)
      continue;

    thing->Update(currentTimeMs, true);

    if (this->isIsolated == false) {
      // Send to all other participants
      for (Participant* participant : Participant::participants) {
        if (participant == nullptr || participant == this)
          continue;

        PoseMsg* poseMsg = new PoseMsg(this->networkId, thing);
        this->Send(participant, poseMsg);
        delete poseMsg;
        BinaryMsg* binaryMsg = new BinaryMsg(this->networkId, thing);
        this->Send(participant, binaryMsg);
        delete binaryMsg;
      }
    }
  }
}

void SiteServer::Process(Participant* sender, ParticipantMsg* msg) {
  if (msg->networkId == 0) {
    // std::cout << this->name << " received New Client -> " <<
    // sender->ipAddress
    //           << ":" << (int)sender->port << "\n";
    SiteMsg* msg = new SiteMsg(sender->networkId);
    this->Send(sender, msg);
    delete msg;
  }
}

void SiteServer::Process(Participant* sender, SiteMsg* msg) {}

void SiteServer::Process(Participant* sender, ThingMsg* msg) {
  Thing* thing = sender->Get(msg->thingId);
  if (thing == nullptr) {
    // #if defined(NO_STD)
    new Thing(sender, (Thing::Type)msg->thingType, msg->thingId);
    // #else
    //     auto thingMsgProcessor = thingMsgProcessors.find(msg->thingType);
    //     //Thing* newThing;
    //     if (thingMsgProcessor != thingMsgProcessors.end())  // found item
    //       //newThing =
    //           thingMsgProcessor->second(sender, msg->networkId,
    //           msg->thingId);
    //     else
    //       //newThing =
    //         new Thing(sender, msg->networkId, msg->thingId,
    //                            (Thing::Type)msg->thingType);
    // #endif
  }
}

}  // namespace RoboidControl
